// ***************************************************************************************
//
//	Copyright (C) 2003 Kazuhiko TAMURA. All rights reserved.
//
//	This program is free software; you can redistribute it and/or 
//	modify it under the terms of the GNU General Public License
//	as published by the Free Software Foundation; either version 2
//	of the License, or (at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//	NAME:		BridgableCheckModel.java
//	DATE:		2003.5.9
//	CREATOR:	Kazuhiko TAMURA
//
// ***************************************************************************************

package jp.gr.java_conf.ktz.puzzle.hashikake.app.model;

import java.awt.Point;

import jp.gr.java_conf.ktz.puzzle.framework.State;
import jp.gr.java_conf.ktz.puzzle.framework.StateEventCode;
import jp.gr.java_conf.ktz.puzzle.framework.Model;
import jp.gr.java_conf.ktz.puzzle.framework.ModelConstants;

import jp.gr.java_conf.ktz.puzzle.hashikake.constants.Direction;

import jp.gr.java_conf.ktz.puzzle.hashikake.fsm.SelectionState;
import jp.gr.java_conf.ktz.puzzle.hashikake.fsm.NumberActiveState;

import jp.gr.java_conf.ktz.puzzle.hashikake.util.UtilityFuncs;

/**
 *	Ֆʂ̑IԂǗModel
 */
public class BridgibleCheckModel extends AbstractDecoratedModel {
	/** ՊÔł߂Ȉʒu萔 */
	private static final Point EMPTY_POS = ModelConstants.ILLEGAL_POS;
	
	/** ύXȂ萔 */
	private static final Point[] NO_MODIFIED = ModelConstants.EMPTIES;
	
	/** Iꂽ̈ʒu */
	private Point mPressedPos = EMPTY_POS;
	private Point mReleasedPos = EMPTY_POS;
	
	/** Iꂽ˂Z̔z */
	private Point[] mSelectedPos = NO_MODIFIED;
	private Point[] mDeactivatedPos = NO_MODIFIED;
	
	/** Model̏CtO */
	private boolean mIsModified = false;
	
	/** Iꂽ */
	private Direction mDirection = Direction.NO;
	
	/**
	 *	RXgN^
	 *	w肵ModelbvModelƂďLB
	 *
	 *	@param	inModel	bvModel
	 */
	public BridgibleCheckModel(Model inModel) {
		super(inModel);
	}
	
	/**
	 *	w肳ꂽʒu̖ʂ̏ԂԂ 
	 *
	 *	@param inX	XW
	 *	@param	inY	YW
	 *	@return 
	 */
	public State getCurStateAt(final int inX, final int inY) {
		final Point aPos = new Point(inX, inY);
		
		State aState = super.getCurStateAt(inX, inY);
		
		if (aPos.equals(mPressedPos)) {
			return new NumberActiveState(aState);
		}
		
		Point[] aModifiedPos = mSelectedPos;
		for (int i = 0; i < aModifiedPos.length; ++i) {
			if (aPos.equals(aModifiedPos[i])) {
				return new SelectionState(aState);
			}
		}
		
		return aState;
	}
		
	/**
	 *	w肳ꂽTCỸ{[h쐬B
	 *	œ̏s
	 *
	 *	@param	inWidth	̌
	 *	@param	inHeight	č
	 */
	public void createBoardSelf(final int inWidth, final int inHeight) {
	}
	
	/**
	 *	ύXꂽՖʂ̈ʒuԂB
	 *	̎ł́Adh悤ɂĂB
	 *
	 *	@return	ύXꂽՖʂ̈ʒȕWB
	 */
	protected Point[] lastModifiedSelf() {
		java.util.Set aSet = new java.util.HashSet();
		
		if (mPressedPos != EMPTY_POS) aSet.add(mPressedPos);
		if (mReleasedPos != EMPTY_POS) aSet.add(mReleasedPos);
				
		aSet.addAll(java.util.Arrays.asList(mDeactivatedPos));
		aSet.addAll(java.util.Arrays.asList(mSelectedPos));
		
		return (Point[])aSet.toArray(new Point[0]);
	}
	
	/**
	 *	ModelύXꂽǂ𒲂ׂB
	 *
	 *	@return	ύXĂtrueԂB
	 */
	protected boolean isModifiedSelf() {
		return mIsModified;
	}
	
	/**
	 *	w肳ꂽStateEventCodẽfŏł邩ǂ`FbNB
	 *
	 *	@param	inCode`FbNStateEventCode
	 *	@return	̃fŏł̂ł΁AtrueԂ
	 */
	public boolean isAcceptableEvent(StateEventCode inCode) {
		return true;
	}

	/**
	 *	IύXB
	 *	̎ł́AZ̃NbNmPressedPosXVA
	 *	}EX̃hbO͑IANeBuԂɍXVB
	 *	܂AIł΁AB
	 *
	 *	@param	ioEvent f̕ύXɊւCxg
	 */
	public void selectionChanged(ModelChangeEvent inEvent) {
		final Direction aDirection = inEvent.getDirection();
		
		// hbOĂȂA܂͂ǂ̕IĂȂ̂ł΁A
		// mPressedPosXVAÏIɂȀŔB
		if (aDirection == Direction.NO) {
			mPressedPos = inEvent.getPos();
			mDeactivatedPos = mSelectedPos;
			mSelectedPos = NO_MODIFIED;
			mDirection = aDirection;
			mIsModified = true;
			
			return;
		}
		
		// IɕύXȂΔ
		if (aDirection == mDirection) return;
		
		mDirection = aDirection;
		
		activate(aDirection);
	}
	
	/**
	 *	w肳ꂽIԂɂ
	 *
	 *	@param	inDirection	IԂɂ
	 */
	private void activate(Direction inDirection) {
		// OɑIꂽʒuIɂB
		mDeactivatedPos = mSelectedPos;
		
		// ˂邱ƂłȂȂAB
		if (! isBuildBridge(inDirection)) {
			mSelectedPos = NO_MODIFIED;
			
			return;
		}
		
		Point aDiff = inDirection.getDifference();
		Point aPos = new Point(mPressedPos);
		java.util.Set aSet = new java.util.HashSet();
		
		aPos.translate(aDiff.x, aDiff.y);
		do {
			aSet.add(new Point(aPos));
			aPos.translate(aDiff.x, aDiff.y);
		} 
		while(! isNumberAt(aPos.x, aPos.y));
		
		mSelectedPos = (Point[])aSet.toArray(new Point[0]);
		
		mIsModified = true;
	}
	
	/**
	 *	w肳ꂽɋ˂邱Ƃł邩ǂ`FbN
	 *
	 *	@param	inDirection	`FbN
	 */
	private boolean isBuildBridge(Direction inDirection) {
		Point aDiff = inDirection.getDifference();
		Point aPos = new Point(mPressedPos);
		
		final StateEventCode aCode = UtilityFuncs.resolveTransitCode(inDirection);
		
		// אڂĂ̂ł΁AfalseԂ
		aPos.translate(aDiff.x, aDiff.y);
		if (! isTransitAt(aPos.x, aPos.y, aCode)) return false;
		
		// ̐ɏo܂ŁAw肳ꂽTB
		// rA󔒈ȊȌԂ邱ƂȂ̐ɏoAfalseԂB
		do {
			aPos.translate(aDiff.x, aDiff.y);
			if (isNumberAt(aPos.x, aPos.y)) return true;
		} while(contains(aPos.x, aPos.y) && isTransitAt(aPos.x, aPos.y, aCode));
		
		// rA󔒈ȊȌԂꂽB
		return false;
	}
	
	/**
	 *	I肷B
	 *	̎ł́AׂĂ̑IB
	 *	fR[gfł̎gp𓥂܂āA
	 *	ύXꂽʒuioEventɃZbgĂ
	 *
	 *	@param	ioEvent f̕ύXɊւCxg
	 */
	public void selectionDetermined(ModelChangeEvent ioEvent) {
		if (mPressedPos.equals(EMPTY_POS)) return;
		
		ioEvent.setModifiedPos(mSelectedPos);
		
		mReleasedPos = mPressedPos;
		
		mPressedPos = EMPTY_POS;
		mSelectedPos = NO_MODIFIED;
		mDirection = Direction.NO;
		
		mIsModified = true;
	}
	
	/**
	 *	Model̕ύXNA
	 */
	protected void flushSelf() {
		mReleasedPos = EMPTY_POS;
		mIsModified = false;
	}
}
